@@ -4,7 +4,7 @@
 
 use crate::grammar::Grammar;
 use crate::grammar::GrammarRule;
-use crate::grammar::Symbol;
+use crate::grammar::ProductionKind;
 use crate::grammar::START_RULE_NAME;
 use crate::lexer::Lexeme;
 use crate::parser::tree::build;
@@ -15,10 +15,10 @@ use crate::parser::Tree;
 use std::collections::HashSet;
 use std::rc::Rc;
 
-fn predict(
-    columns: &mut Vec<ParserColumn>,
+fn predict<Value>(
+    columns: &mut Vec<ParserColumn<Value>>,
     column_index: usize,
-    rule: &GrammarRule,
+    rule: &GrammarRule<Value>,
 ) {
     for production in &rule.productions {
         if column_index + 1 < columns.len()
@@ -42,8 +42,8 @@ fn predict(
     }
 }
 
-fn scan(
-    columns: &mut Vec<ParserColumn>,
+fn scan<Value>(
+    columns: &mut Vec<ParserColumn<Value>>,
     column_index: usize,
     state_index: usize,
 ) {
@@ -58,8 +58,8 @@ fn scan(
     columns[column_index + 1].add(new_state);
 }
 
-fn complete(
-    columns: &mut Vec<ParserColumn>,
+fn complete<Value>(
+    columns: &mut Vec<ParserColumn<Value>>,
     column_index: usize,
     state_index: usize,
 ) {
@@ -71,15 +71,21 @@ fn complete(
         .states
         .iter()
         .enumerate()
-        .filter_map(|(index, st)| match st.next_symbol() {
-            Some(Symbol::Rule(name)) => {
-                if name == **state_name {
-                    Some(index)
-                } else {
-                    None
+        .filter_map(|(index, st)| {
+            if let ProductionKind::Rules = st.production.kind {
+                match st.next_symbol() {
+                    Some(rule_name) => {
+                        if *rule_name == **state_name {
+                            Some(index)
+                        } else {
+                            None
+                        }
+                    }
+                    _ => None,
                 }
+            } else {
+                None
             }
-            _ => None,
         })
         .collect();
 
@@ -97,11 +103,11 @@ fn complete(
 }
 
 /// Parse the provided (Lexemes)(Lexeme) with the given [Grammar]
-pub fn parse(
-    grammar: &Grammar,
+pub fn parse<Value>(
+    grammar: &Grammar<Value>,
     lexemes: &[Lexeme],
-) -> Result<Vec<Rc<Tree>>, ParseError> {
-    let mut columns: Vec<ParserColumn> = earley(grammar, lexemes);
+) -> Result<Vec<Rc<Tree<Value>>>, ParseError<Value>> {
+    let mut columns: Vec<ParserColumn<Value>> = earley(grammar, lexemes);
 
     let mut parent = None;
     for state in &columns.last().unwrap().states {
@@ -140,8 +146,11 @@ pub fn parse(
 
 /// Parse the provided [Lexemes](Lexeme) with the given [Grammar]
 /// and the [Earley algorithm](https://en.wikipedia.org/wiki/Earley_parser).
-pub fn earley(grammar: &Grammar, lexemes: &[Lexeme]) -> Vec<ParserColumn> {
-    let mut columns: Vec<ParserColumn> = (0..=lexemes.len())
+pub fn earley<Value>(
+    grammar: &Grammar<Value>,
+    lexemes: &[Lexeme],
+) -> Vec<ParserColumn<Value>> {
+    let mut columns: Vec<ParserColumn<Value>> = (0..=lexemes.len())
         .map(|index| {
             if index == 0 {
                 ParserColumn {
@@ -182,17 +191,19 @@ pub fn earley(grammar: &Grammar, lexemes: &[Lexeme]) -> Vec<ParserColumn> {
             if columns[column_index].states[state_index].completed() {
                 complete(&mut columns, column_index, state_index);
             } else {
-                match state.next_symbol().unwrap() {
-                    Symbol::Rule(name) => {
-                        if !predicted_names.contains(&name) {
-                            let rule = grammar.rules.get(&name).unwrap();
+                match state.production.kind {
+                    ProductionKind::Rules => {
+                        let rule_name = state.next_symbol().unwrap().clone();
+                        if !predicted_names.contains(&rule_name) {
+                            let rule = grammar.rules.get(&rule_name).unwrap();
+                            predicted_names.insert(rule_name);
                             predict(&mut columns, column_index, rule);
-                            predicted_names.insert(name);
                         }
                     }
-                    Symbol::Lexeme(kind) => {
+                    ProductionKind::Lexemes => {
                         if column_index + 1 < columns.len()
-                            && kind == columns[column_index + 1].kind
+                            && *state.next_symbol().unwrap()
+                                == columns[column_index + 1].kind
                         {
                             scan(&mut columns, column_index, state_index);
                         }
