@@ -3,8 +3,10 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 pub mod ambiguous_integer_addition;
+pub mod calculator;
 pub mod integer_addition;
 pub mod javascript_string_interpolation;
+pub mod nix;
 pub mod smallest;
 
 #[test]
@@ -16,6 +18,15 @@ fn ambiguous_integer_addition() {
     );
 }
 
+#[test]
+fn calculator() {
+    run(
+        "calculator",
+        &calculator::lexer::lexer_rules(),
+        &calculator::grammar::grammar(),
+    );
+}
+
 #[test]
 fn javascript_string_interpolation() {
     run(
@@ -35,21 +46,8 @@ fn integer_addition() {
 }
 
 #[test]
-fn language_calculator() {
-    run(
-        "language_calculator",
-        &santiago::languages::calculator::lexer_rules(),
-        &santiago::languages::calculator::grammar(),
-    );
-}
-
-#[test]
-fn language_nix() {
-    run(
-        "language_nix",
-        &santiago::languages::nix::lexer_rules(),
-        &santiago::languages::nix::grammar(),
-    );
+fn nix() {
+    run("nix", &nix::lexer::lexer_rules(), &nix::grammar::grammar());
 }
 
 #[test]
@@ -61,10 +59,10 @@ fn smallest() {
     );
 }
 
-fn run(
+fn run<Value>(
     name: &str,
     lexer_rules: &santiago::lexer::LexerRules,
-    grammar: &santiago::grammar::Grammar,
+    grammar: &santiago::grammar::Grammar<Value>,
 ) {
     use std::io::Write;
     let should_update = std::env::var("UPDATE").is_ok();
